#include "nokia_3310_lcd.h"
#include "avr_bmp.h"
#include<stdlib.h>	//allows use of dtostrf

/* M. Suzuki, mod to Nokia code for
 * HSMG20 temperature/RH sensor
 */


/*
  HGM20G Rev.1
  http://sites.google.com/site/measuringstuff

 */
const byte nsamp=10;

int humidityPin=1;
int Thgm20Pin=2;
unsigned int sensorValue2 = 0;  // value coming from the sensor
unsigned int sensorValue3 = 0;  // value coming from the sensor
float TinF=0.0;

//keypad debounce parameter
#define DEBOUNCE_MAX 15
#define DEBOUNCE_ON  10
#define DEBOUNCE_OFF 3 

#define NUM_KEYS 5

#define NUM_MENU_ITEM	4

// joystick number
#define UP_KEY 3
#define LEFT_KEY 0
#define CENTER_KEY 1
#define DOWN_KEY 2
#define RIGHT_KEY 4

// menu starting points

#define MENU_X	10		// 0-83
#define MENU_Y	1		// 0-5



// adc preset value, represent top value,incl. noise & margin,that the adc reads, when a key is pressed
// set noise & margin = 30 (0.15V@5V)
int  adc_key_val[5] ={30, 150, 360, 535, 760 };

// debounce counters
byte button_count[NUM_KEYS];
// button status - pressed/released
byte button_status[NUM_KEYS];
// button on flags for user program 
byte button_flag[NUM_KEYS];

// menu definition
char menu_items[NUM_MENU_ITEM][12]={
	"TEMP_RH",
	"CHAR MAP",
	"BITMAP",
	"ABOUT"	
};

void (*menu_funcs[NUM_MENU_ITEM])(void) = {
	tempRH,
	charmap,
	bitmap,
	about
};

char current_menu_item;


Nokia_3310_lcd lcd=Nokia_3310_lcd();

void setup()
{
    
   // setup interrupt-driven keypad arrays  
   // reset button arrays
   for(byte i=0; i<NUM_KEYS; i++){
     button_count[i]=0;
     button_status[i]=0;
     button_flag[i]=0;
   }
  
  // Setup timer2 -- Prescaler/256
  TCCR2A &= ~((1<<WGM21) | (1<<WGM20));
  TCCR2B &= ~(1<<WGM22);
  TCCR2B = (1<<CS22)|(1<<CS21);      
  
  ASSR |=(0<<AS2);

   // Use normal mode  
   TCCR2A =0;    
     //Timer2 Overflow Interrupt Enable  
     TIMSK2 |= (0<<OCIE2A);
     TCNT2=0x6;  // counting starts from 6;  
   TIMSK2 = (1<<TOIE2);    
   
                  
 
  SREG|=1<<SREG_I;
  
  lcd.init();
  lcd.clear();
  
   //menu initialization
   init_MENU();
   current_menu_item = 0;	
}


/* loop */

void loop()
{


    byte i;
    for(i=0; i<NUM_KEYS; i++){
     if(button_flag[i] !=0){
            
        button_flag[i]=0;  // reset button flag
	  switch(i){

		case UP_KEY:
			// current item to normal display
			lcd.writeString(MENU_X, MENU_Y + current_menu_item, menu_items[current_menu_item], MENU_NORMAL );
			current_menu_item -=1;
			if(current_menu_item <0)  current_menu_item = NUM_MENU_ITEM -1;
			// next item to highlight display
			lcd.writeString(MENU_X, MENU_Y + current_menu_item, menu_items[current_menu_item], MENU_HIGHLIGHT );
			break;
		case DOWN_KEY:
			// current item to normal display
			lcd.writeString(MENU_X, MENU_Y + current_menu_item, menu_items[current_menu_item], MENU_NORMAL );
			current_menu_item +=1;
			if(current_menu_item >(NUM_MENU_ITEM-1))  current_menu_item = 0;
			// next item to highlight display
			lcd.writeString(MENU_X, MENU_Y + current_menu_item, menu_items[current_menu_item], MENU_HIGHLIGHT );
			break;
		case LEFT_KEY:
          		init_MENU();
			current_menu_item = 0;
			break;   
		case RIGHT_KEY:
        	lcd.clear();
		(*menu_funcs[current_menu_item])();
          lcd.clear();
	init_MENU();
		current_menu_item = 0;           
		break;	
			}
				
		}
    }

}

/* menu functions */

void init_MENU(void){

  byte i;
  
  lcd.clear();

    lcd.writeString(MENU_X, MENU_Y, menu_items[0], MENU_HIGHLIGHT );
	
  for (i=1; i<NUM_MENU_ITEM; i++){
    lcd.writeString(MENU_X, MENU_Y+i, menu_items[i], MENU_NORMAL);
  }
	
	
}

// waiting for center key press
void waitfor_OKkey(){
  byte i;
  byte key = 0xFF;
	while (key!= CENTER_KEY){
    for(i=0; i<NUM_KEYS; i++){
       if(button_flag[i] !=0){
           button_flag[i]=0;  // reset button flag
           if(i== CENTER_KEY) key=CENTER_KEY;
        }
     }
   }
		
}

void tempRH()  // Temp RH stuff
{
  while(1)	//Do until reset or power off
  {
  	for (byte j=0;j<nsamp;j++)
  	{    
    		sensorValue2 += analogRead(humidityPin);    
    		sensorValue3 += analogRead(Thgm20Pin); 
  	}    
  	int sensorValue2Avg=sensorValue2/nsamp;

	// original
  	float RH= 0.1515*sensorValue2Avg-12.0;


/*	//modified
	float RH=0.104242*sensorValue2Avg-12.0;  
*/

	// original
  	int sensorValue3Avg=sensorValue3/nsamp;
  	float Vt=(float) sensorValue3Avg*5.0/1023.0;
	float R=(5.0-Vt)*10.0/Vt;
	float TinC=281.583*pow(1.0230,(1.0/R))*pow(R,-0.1227)-150.6614; 
	TinF=TinC*(9.0/5.0)+32; 

/*
	//modified for 3.44v ARef
  	int sensorValue3Avg=sensorValue3/nsamp;
  	float Vt=(float) sensorValue3Avg*3.44/1023.0;
	float R=(3.44-Vt)*10.0/Vt;
	float TinC=281.583*pow(1.0230,(1.0/R))*pow(R,-0.1227)-150.6614; 
	TinF=TinC*(9.0/5.0)+32; 
*/



	// Temp RH stuff

	char floatBuffer[12]; //buffer for dtostrf() function

	dtostrf(TinF,4,1,floatBuffer);	//converts float to string
	lcd.writeString(0, 1,floatBuffer, MENU_NORMAL );
	lcd.writeString(24, 1," DegF", MENU_NORMAL );
	dtostrf(RH,2,0,floatBuffer);
	lcd.writeString(6, 3,floatBuffer, MENU_NORMAL );
	lcd.writeString(20, 3, " %RH", MENU_NORMAL );
    	sensorValue2=0;
    	sensorValue3=0;
    	delay(5000);
   }
}

void charmap(){
/* MTS
  char i,j;
   for(i=0; i<5; i++){
    for(j=0; j<14; j++){
      lcd.LCD_3310_set_XY(j*6,i);
      lcd.LCD_3310_write_char(i*14+j+32, MENU_NORMAL);
      }
    }
  
*/
  
  lcd.writeString(38, 5, "OK", MENU_HIGHLIGHT );
	waitfor_OKkey();   
}

void bitmap(){
  lcd.drawBitmap(20,1, AVR_bmp, 48,24);
  lcd.writeString(38, 5, "OK", MENU_HIGHLIGHT );
	waitfor_OKkey();
}


void about(){
  
  lcd.writeString( 0, 1, "Nokia 3310 LCD", MENU_NORMAL);
  lcd.writeString( 0, 3, "nuelectronics", MENU_NORMAL);
	lcd.writeString(38, 5, "OK", MENU_HIGHLIGHT );
	waitfor_OKkey();
	
		
}



// The followinging are interrupt-driven keypad reading functions
//  which includes DEBOUNCE ON/OFF mechanism, and continuous pressing detection


// Convert ADC value to key number
char get_key(unsigned int input)
{
	char k;
    
	for (k = 0; k < NUM_KEYS; k++)
	{
		if (input < adc_key_val[k])
		{
           
    return k;
        }
	}
    
    if (k >= NUM_KEYS)
        k = -1;     // No valid key pressed
    
    return k;
}

void update_adc_key(){
  int adc_key_in;
  char key_in;
  byte i;
  
  adc_key_in = analogRead(0);
  key_in = get_key(adc_key_in);
  for(i=0; i<NUM_KEYS; i++)
  {
    if(key_in==i)  //one key is pressed 
    { 
      if(button_count[i]<DEBOUNCE_MAX)
      {
        button_count[i]++;
        if(button_count[i]>DEBOUNCE_ON)
        {
          if(button_status[i] == 0)
          {
            button_flag[i] = 1;
            button_status[i] = 1; //button debounced to 'pressed' status
          }
		  
        }
      }
	
    }
    else // no button pressed
    {
      if (button_count[i] >0)
      {  
		button_flag[i] = 0;	
		button_count[i]--;
        if(button_count[i]<DEBOUNCE_OFF){
          button_status[i]=0;   //button debounced to 'released' status
        }
      }
    }
    
  }
}

// Timer2 interrupt routine -
// 1/(160000000/256/(256-6)) = 4ms interval

ISR(TIMER2_OVF_vect) {  
  TCNT2  = 6;
  update_adc_key();
}



